# This file was generated by liblab | https://liblab.com/

from typing import List, Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.environment.environment import Environment
from ..models.utils.sentinel import SENTINEL
from ..models.utils.cast_models import cast_models
from ..models import HttpValidationError, Value


class LayoutsService(BaseService):

    @cast_models
    def new(
        self, init: Union[str, None] = SENTINEL, version: Union[str, None] = SENTINEL
    ) -> any:
        """Create new version (default or specified settings)

        :param init: init, defaults to None
        :type init: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(init)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/revision/new",
                [self.get_api_key()],
            )
            .add_query("init", init, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def get(self, name: str, version: Union[str, None] = SENTINEL) -> str:
        """Obtain the lastest layout for page with specified 'name'

        :param name: name
        :type name: str
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: str
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/actual/get",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def set(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all previous layouts for specified page with 'name' and add a new value

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/actual/set",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def add(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
    ) -> any:
        """Add a new value for specified page by 'name'

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/actual/add",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def all(
        self, name: Union[str, None] = SENTINEL, version: Union[str, None] = SENTINEL
    ) -> List[Value]:
        """Obtain a list of all layouts with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: List[Value]
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/all/get",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return [Value._unmap(item) for item in response]

    @cast_models
    def update(
        self, request_body: List[Value], version: Union[str, None] = SENTINEL
    ) -> any:
        """Remove previously set and add new layouts with specified 'name' fileds with layouts from 'layouts' fileds of provided list

        :param request_body: The request body.
        :type request_body: List[Value]
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(Value).is_array().validate(request_body)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/all/update",
                [self.get_api_key()],
            )
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def remove(
        self,
        name: Union[str, None] = SENTINEL,
        value: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all layouts for specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/all/remove",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("DELETE")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def display(
        self,
        name: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
        format: Union[str, None] = SENTINEL,
    ) -> any:
        """Display a list of all layouts with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        :param format: format, defaults to None
        :type format: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(version)
        Validator(str).is_optional().is_nullable().validate(format)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/all/display",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("version", version, nullable=True)
            .add_query("format", format, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def revisions(self) -> any:
        """List of all avaliable revisions (aka 'versions')

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/layouts/rev/list",
                [self.get_api_key()],
            )
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def new_1(
        self, init: Union[str, None] = SENTINEL, version: Union[str, None] = SENTINEL
    ) -> any:
        """Create new version (default or specified settings)

        :param init: init, defaults to None
        :type init: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(init)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/revision/new",
                [self.get_api_key()],
            )
            .add_query("init", init, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def get_1(self, name: str, version: Union[str, None] = SENTINEL) -> str:
        """Obtain the lastest layout for page with specified 'name'

        :param name: name
        :type name: str
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: str
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/actual/get",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def set_1(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all previous layouts for specified page with 'name' and add a new value

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/actual/set",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def add_1(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
    ) -> any:
        """Add a new value for specified page by 'name'

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/actual/add",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def all_1(
        self, name: Union[str, None] = SENTINEL, version: Union[str, None] = SENTINEL
    ) -> List[Value]:
        """Obtain a list of all layouts with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: List[Value]
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/all/get",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return [Value._unmap(item) for item in response]

    @cast_models
    def update_1(
        self, request_body: List[Value], version: Union[str, None] = SENTINEL
    ) -> any:
        """Remove previously set and add new layouts with specified 'name' fileds with layouts from 'layouts' fileds of provided list

        :param request_body: The request body.
        :type request_body: List[Value]
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(Value).is_array().validate(request_body)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/all/update",
                [self.get_api_key()],
            )
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def remove_1(
        self,
        name: Union[str, None] = SENTINEL,
        value: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all layouts for specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/all/remove",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("DELETE")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def display_1(
        self,
        name: Union[str, None] = SENTINEL,
        version: Union[str, None] = SENTINEL,
        format: Union[str, None] = SENTINEL,
    ) -> any:
        """Display a list of all layouts with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        :param format: format, defaults to None
        :type format: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(version)
        Validator(str).is_optional().is_nullable().validate(format)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/all/display",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("version", version, nullable=True)
            .add_query("format", format, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def revisions_1(self) -> any:
        """List of all avaliable revisions (aka 'versions')

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/layouts/rev/list",
                [self.get_api_key()],
            )
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response
